import datetime
from bson import ObjectId

from pydantic import BaseModel
from utils.db import mongo_database

class _Prompt(BaseModel):
    """
    The data of a prompt consisting of the prompt, timestamp and job id.
    """
    prompt: str = ""
    timestamp: datetime.datetime = datetime.datetime.now()
    job_id: str = ""

# function to save promptt to database with timestamp
def save_prompt(data: _Prompt):
    """
    Save prompt to database with timestamp.
    
    :param prompt: Prompt to save.
    :type prompt: str
    :return: Empty response.
    :rtype: dict
    """
    # save prompts to database
    result = mongo_database["Prompts"].insert_one(data.dict())

    # update prompt id in job table
    mongo_database["Job"].update_one(
        {"_id": ObjectId(data.job_id)},
        {"$push": {"prompts": str(result.inserted_id)}}
    )

    return {"id": str(result.inserted_id)}

def get_prompts(job_id: str):
    """
    Get all prompts for a job.
    
    :param job_id: ID of the job to get prompts for.
    :type job_id: str
    :return: List of prompts.
    :rtype: list
    """
    prompts = mongo_database["Job"].find_one({"_id": ObjectId(job_id)})["prompts"]

    return list(prompts)

def get_prompt_details(prompt_id: str):
    """
    Get details of a prompt.
    
    :param prompt_id: ID of the prompt to get details for.
    :type prompt_id: str
    :return: Details of the prompt.
    :rtype: dict
    """
    prompt = mongo_database["Prompts"].find_one({"_id": ObjectId(prompt_id)})

    return prompt


from llama_index.vector_stores.mongodb import MongoDBAtlasVectorSearch

import os
import logging
import pymongo

def get_mongo_store():
    try:
        logger = logging.getLogger("uvicorn")
        mongo_client = get_mongo_client()
        store = MongoDBAtlasVectorSearch(
                mongo_client,
                db_name=os.environ["MONGODB_DATABASE"],
                collection_name=os.environ["MONGODB_VECTORS"],
                index_name=os.environ["MONGODB_VECTOR_INDEX"],
            )
        return store
    except Exception as e:
        logger.error(f"Error while connecting to index: {e}")
        # Handle the error as required
        return None 

def get_mongo_client():
    """Establish connection to the MongoDB."""
    try:
        mongo_uri = os.environ["MONGO_URI"]
        client = pymongo.MongoClient(mongo_uri)
        print("Connection to MongoDB successful")
        return client
    except pymongo.errors.ConnectionFailure as e:
        print(f"Connection failed: {e}")
        return None





from fastapi import FastAPI, HTTPException, APIRouter, Depends,Request
from pydantic import BaseModel
import stripe
import os
import logging
from utils.db import mongo_database

# Initialize logging
logging.basicConfig(level=logging.INFO)
logger = logging.getLogger(__name__)

# Initialize Stripe with your secret key
stripe.api_key = os.getenv("STRIPE_SECRET_KEY")

# Initialize FastAPI and MongoDB client
app = FastAPI()

stripe_session_router = APIRouter()

class CheckoutSessionRequest(BaseModel):
    price_id: str
    email: str

@stripe_session_router.post("/create-checkout-session")
async def create_checkout_session(request: Request):
    logger.info("Received request to create checkout session")
    # logger.info(f"Request data: {request}")
    data=await request.json()
    # print(data)
    email=data['email']
    price_id=data['price_id']
    

    # Fetch the customer ID from MongoDB
    try:
        user_data = mongo_database["Users"].find_one({"email": email})
        if not user_data:
            logger.error(f"User with email {email} not found")
            raise HTTPException(status_code=404, detail="User not found")
    except Exception as e:
        logger.error(f"Error fetching user data: {e}")
        raise HTTPException(status_code=500, detail="Error fetching user data")

    customer_id = user_data.get("stripe_customer_id")

    logger.info(f"Fetched customer ID: {customer_id}")
    success_url=os.getenv("PAYMENT_SUCCESS_URL")
    cancel_url=os.getenv("PAYMENT_CANCEL_URL")
    # Create a Stripe Checkout Session
    try:
        checkout_session = stripe.checkout.Session.create(
            customer=customer_id,
            payment_method_types=["card"],
            line_items=[
                {
                    "price": price_id,
                    "quantity": 1,
                },
            ],
            mode="subscription",
            success_url=success_url,
            cancel_url=cancel_url,
        )
        logger.info(f"Created checkout session: {checkout_session['id']}")
        return {"id": checkout_session["id"]}
    
    except Exception as e:
        logger.error(f"Error creating checkout session: {e}")
        raise HTTPException(status_code=500, detail="Error creating checkout session")



from dotenv import load_dotenv

load_dotenv()
import boto3
import os

# Initialize AWS S3 client
s3_client = boto3.client('s3', 
                  aws_access_key_id=os.environ["AWS_ACCESS_KEY_ID"], 
                  aws_secret_access_key=os.environ["AWS_SECRET"], 
                  region_name=os.environ["AWS_REGION"])

lambda_client = boto3.client('lambda',
                            aws_access_key_id=os.environ["AWS_ACCESS_KEY_ID"], 
                            aws_secret_access_key=os.environ["AWS_SECRET"], 
                            region_name=os.environ["AWS_REGION"])



from app.engine.index import get_index

print('Initializing package...')

def get_chat_engine():
    return get_index().as_chat_engine(
        similarity_top_k=3, chat_mode="condense_plus_context"
    )



import csv
import os
import uuid
# async def convert_list_to_csv(csv_file_name: str, column_names:list[str], all_failed_downloads: list[str]):

#     unique_id = uuid.uuid4()
#     temp_dir = os.path.join(os.path.expanduser("~"),f"{unique_id}")
#     if not os.path.exists(temp_dir):
#         os.makedirs(temp_dir)

#     # Convert failed_files to CSV and save it to a file
#     csv_path = os.path.join(temp_dir, csv_file_name)
        
#     with open(csv_path, mode='w', newline='') as csv_file:
#         writer = csv.writer(csv_file)
#         writer.writerow(column_names)
#         writer.writerows(all_failed_downloads)

#     attachments=list[str]
#     if(len(all_failed_downloads)>=1): 
#         attachments.append(csv_path)

#     return attachments  

async def convert_list_to_csv(csv_file_name: str, column_names: list[str], all_failed_downloads: list[list[str]]) -> list[str]:
    unique_id = uuid.uuid4()
    temp_dir = os.path.join(os.path.expanduser("~"), f"{unique_id}")
    
    if not os.path.exists(temp_dir):
        os.makedirs(temp_dir)

    # Convert failed_files to CSV and save it to a file
    csv_path = os.path.join(temp_dir, csv_file_name)
    
    with open(csv_path, mode='w', newline='') as csv_file:
        writer = csv.writer(csv_file)
        writer.writerow(column_names)
        writer.writerows(all_failed_downloads)

    # Initialize attachments list and append csv_path if needed
    attachments = []
    if len(all_failed_downloads) >= 1:
        attachments.append(csv_path)

    return attachments


# Helper function to convert ObjectId to string
def object_id_to_str(data):
    if "_id" in data:
        data["_id"] = str(data["_id"])
    return data


from fastapi import APIRouter, HTTPException, Body, Path
from pydantic import BaseModel, Field
from typing import List
from bson import ObjectId
from utils.db import mongo_database

template_router = t = APIRouter()

# MongoDB Setup
template_collection = mongo_database["templates"]
variable_collection = mongo_database["variables"]

# Helper function to convert MongoDB ObjectId to string
def to_dict(document):
    """Convert MongoDB document to a dictionary with stringified `_id`."""
    document["_id"] = str(document["_id"])
    return document

# Pydantic Models
class Template(BaseModel):
    id: str = Field(None, alias="_id")  # Use alias to map `_id` field from MongoDB
    name: str = Field(..., title="Template Name", max_length=100)
    subject: str = Field(..., title="Template Subject", max_length=200)
    content: str = Field(..., title="Template Content")

    class Config:
        # allow_population_by_field_name = True  # Allow `_id` to be included in the output as `_id`
        populate_by_name = True  


class TemplateUpdate(BaseModel):
    name: str | None = Field(None, title="Template Name", max_length=100)
    subject: str | None = Field(None, title="Template Subject", max_length=200)
    content: str | None = Field(None, title="Template Content")

class Variable(BaseModel):
    _id: str
    name: str = Field(..., title="Variable Name", max_length=100)

@t.get("/variables", response_model=List[Variable], summary="Fetch all predefined variables")
async def get_variables():
    variables = list(variable_collection.find())
    return [to_dict(variable) for variable in variables]

@t.post("/variables/", response_model=Variable, summary="Create a new predefined variable")
async def create_variable(variable: Variable):
    variable_dict = variable.dict()
    result = variable_collection.insert_one(variable_dict)
    if result.inserted_id:
        return to_dict({**variable_dict, "_id": result.inserted_id})
    raise HTTPException(status_code=500, detail="Failed to create variable")

# Routes
@t.get("/", response_model=List[Template], summary="Fetch all templates")
async def get_templates():
    templates = list(template_collection.find())
    # Convert ObjectId to string for JSON serialization
    for template in templates:
        if "_id" in template:
            template["_id"] = str(template["_id"])
    return templates


@t.get("/{id}", response_model=Template, summary="Fetch a template by ID")
async def get_template_by_id(
    id: str = Path(..., title="Template ID", description="ID of the template to fetch")
):
    if not ObjectId.is_valid(id):
        raise HTTPException(status_code=400, detail="Invalid template ID")
    template = template_collection.find_one({"_id": ObjectId(id)})
    if not template:
        raise HTTPException(status_code=404, detail="Template not found")
    return to_dict(template)

@t.post("/", response_model=Template, summary="Create a new template")
async def create_template(template: Template):
    template_dict = template.dict(exclude={"_id"})  # Exclude _id on create
    result = template_collection.insert_one(template_dict)
    if result.inserted_id:
        return to_dict({**template_dict, "_id": result.inserted_id})
    raise HTTPException(status_code=500, detail="Failed to create template")

@t.put("/{id}", response_model=Template, summary="Edit an existing template")
async def edit_template(
    id: str = Path(..., title="Template ID", description="ID of the template to edit"),
    update_data: TemplateUpdate = Body(...)
):
    if not ObjectId.is_valid(id):
        raise HTTPException(status_code=400, detail="Invalid template ID")
    update_data_dict = {k: v for k, v in update_data.dict().items() if v is not None}
    if not update_data_dict:
        raise HTTPException(status_code=400, detail="No valid fields to update")
    result = template_collection.find_one_and_update(
        {"_id": ObjectId(id)},
        {"$set": update_data_dict},
        return_document=True
    )
    if result:
        return to_dict(result)
    raise HTTPException(status_code=404, detail="Template not found")

@t.delete("/{id}", summary="Delete a template")
async def delete_template(
    id: str = Path(..., title="Template ID", description="ID of the template to delete")
):
    if not ObjectId.is_valid(id):
        raise HTTPException(status_code=400, detail="Invalid template ID")
    result = template_collection.delete_one({"_id": ObjectId(id)})
    if result.deleted_count:
        return {"message": "Template deleted successfully"}
    raise HTTPException(status_code=404, detail="Template not found")




from fastapi import APIRouter, HTTPException, status
from bson import ObjectId
from app.models import _Company, Company
from utils.db import mongo_database
from datetime import datetime
import traceback
# Create an instance of APIRouter to handle company-related routes
company_router = c = APIRouter()

# Endpoint to create a new company
@c.post("/{user_id}")
async def create_company(
    user_id: str,
    data: _Company,
):
    """
    Create a new company.

    :param data: Data of the company to be created and the user id.
    :type data: str, _Company
    :raises HTTPException 400: If no company name is provided.
    :return: ID of the created company.
    :rtype: dict
    """
    # check preconditions and get last message
    if data.company_name == "":
        raise HTTPException(
            status_code=status.HTTP_400_BAD_REQUEST,
            detail="No company provided",
        )
    
    # check if user exists
    user_data = mongo_database["Users"].find_one({"_id": ObjectId(user_id)})
    
    if not user_data:
        raise HTTPException(
            status_code=status.HTTP_404_NOT_FOUND,
            detail="User not found"
        )
    # Define your query
    # query = {"no_of_companies": {"$exists": True}}

    # Check if the field exists in any document in the collection
    if user_data and "user_companies_count" in user_data:
        current_companies_count = user_data["user_companies_count"]
    else:
        current_companies_count = 0

    current_companies_count += 1

    result = mongo_database["Company"].insert_one({
        **data.dict(),
        "is_deleted": False,
        "created_at": datetime.now(),  
        "updated_at": datetime.now() 
    })

    company_id = str(result.inserted_id)

    mongo_database["Users"].update_one(
        {"_id": ObjectId(user_id)}, 
        {
            # "$push": {"company": {"company": company_id}},
            "$push": {"companies": str(company_id)},
            "$set": {"user_companies_count": current_companies_count,"updated_at": datetime.now()}
        }
    )
    
    return {"id": company_id}

# Endpoint to retrieve all companies associated with a user
@c.get("/{user_id}/all")
async def get_all_companies(
    user_id: str,
):
    """
    Retrieve all companies associated with a user.

    :param user_id: ID of the user.
    :type user_id: str
    :return: List of companies associated with the user.
    :rtype: dict
    """
    user_details = mongo_database["Users"].find_one({"_id": ObjectId(user_id)})
    if not user_details or "companies" not in user_details:
        # Handle the case where the user or their associated companies are not found
        raise HTTPException(
            status_code=status.HTTP_404_NOT_FOUND,
            detail="No companies associated with this user"
        )

    # company_ids = user_details["company"]
    company_ids = user_details["companies"]
    # return {}

    companies = []
    for company_id in company_ids:
        company_dict = mongo_database["Company"].find_one({"_id": ObjectId(company_id)})
        if company_dict and (company_dict["is_deleted"] is None or company_dict["is_deleted"]==False) :
            company = Company(company_dict["_id"], company_dict["company_name"], company_dict["is_deleted"])
            companies.append(company)
 
    # Convert ObjectId to string for JSON serialization
    for company in companies:
        company._id = str(company._id)
      
    return companies
    
# Endpoint to retrieve jobs of a specific company
@c.get("/{id}/jobs")
async def get_jobs(
    id: str,
):
    """
    Retrieve jobs of a specific company.

    :param id: ID of the company.
    :type id: str
    :return: List of jobs of the company.
    :rtype: dict
    """
    try:
        company_id = (id)
        company = mongo_database["Company"].find_one({"_id": ObjectId(company_id)})

        if company is None:
            raise HTTPException(status_code=404, detail="Company not found")

        jobs = company.get("jobs", [])

        job_ids = []
        for job_id in jobs:
            job = mongo_database["Job"].find_one({"_id": ObjectId(job_id)})
            if job and not job.get("is_deleted", False):
                job_ids.append({"_id": str(job_id)})

        return job_ids

    except Exception as e:
        traceback.print_exc()
        raise HTTPException(status_code=500, detail=f"Error retrieving jobs: {str(e)}")

@c.delete("/{id}")
def delete_company(id: str):
    """
    Delete a specific company and update the associated user's company and job counts.

    :param id: ID of the company.
    :type id: str
    :return: {"message": "Company Deleted successfully"}
    :rtype: dict
    :raises HTTPException 404: If the company or user is not found.
    """
    company_id = id
    company = mongo_database["Company"].find_one({"_id": ObjectId(company_id)})
    jobs = company.get("jobs", [])

    try:
        # Retrieve the user document containing the company_id
        user_doc = mongo_database["Users"].find_one({"companies": company_id})

        # Check if the user document exists
        if user_doc:
            # Decrement the user_companies_count
            new_companies_count = user_doc["user_companies_count"] - 1

            # Calculate the number of jobs to be removed from user_jobs_count
            jobs_to_remove_count = len(jobs)

            # Update the user's document with the new counts
            mongo_database["Users"].update_one(
                {"_id": user_doc["_id"]},
                {
                    "$set": {
                        "user_companies_count": new_companies_count,
                        "user_jobs_count": user_doc["user_jobs_count"] - jobs_to_remove_count,
                        "updated_at": datetime.now()
                    }
                }
            )
        else:
            raise HTTPException(
                status_code=status.HTTP_404_NOT_FOUND,
                detail="User not found"
            )

        # Delete the company and its associated jobs
        for job in jobs:
            job_id = str(job)
            mongo_database["Job"].update_one(
                {"_id": ObjectId(job_id)},
                {"$set": {"is_deleted": True, "updated_at": datetime.now()}}
            )

        mongo_database["Company"].update_one(
            {"_id": ObjectId(company_id)},
            {"$set": {"is_deleted": True, "updated_at": datetime.now()}}
        )

        return {"message": "Company Deleted successfully"}

    except Exception as e:
        raise HTTPException(
            status_code=status.HTTP_404_NOT_FOUND,
            detail=str(e)
        )


from dotenv import load_dotenv
load_dotenv()
import os
from llama_index.llms.openai import OpenAI

# finetuning_handler = OpenAIFineTuningHandler()
# callback_manager = CallbackManager([finetuning_handler])

llm = OpenAI(model="gpt-4", temperature=0)



from fastapi import APIRouter, Depends, HTTPException, status,Request
from pydantic import BaseModel, EmailStr
from bson import ObjectId
from bson.json_util import dumps
import json
from typing import Optional
from datetime import datetime, timedelta
from urllib.parse import unquote

from utils.db import mongo_database
from urllib.parse import unquote
from fastapi.responses import JSONResponse
# from fastapi_mail import FastMail, MessageSchema,ConnectionConfig
import smtplib
from dotenv import load_dotenv
import os
load_dotenv()
import traceback
import chargebee
import os

from datetime import datetime

# Create an instance of APIRouter to handle user-related routes
user_router = u = APIRouter()

class _UserCreateRequest(BaseModel):
    email: str
    plan: str

class _User(BaseModel):
    #The data of a user consisting of the user email and plan.
    id: str
    email: EmailStr
    plan: str
    end_date: datetime
    start_date: datetime
    user_companies_count: Optional[int] = None
    user_jobs_count: Optional[int] = None
    
class _Contact(BaseModel):
    #The data of a candidate consisting of the candidate email and contact.
    # uid: Optional[str]
    email: str
    contact: Optional[str] = None

class SubscriptionRequest(BaseModel):
    user_id: str
    plan_type: str
    frequency: str

class RenewalRequest(BaseModel):
    user_id: str
    frequency: str


# Endpoint to check if a user exists
@u.get("/exists/{email}", response_model=_User)
async def check_user(email: str):
    """
    Check if a user exists.

    :param email: Email of the user to be checked.
    :type email: str
    :return: User data if the user exists.
    :rtype: dict
    """
    
    try:    
        # Check if user exists
        email=unquote(email)
        user_data = mongo_database["Users"].find_one({"email": email})
        if not user_data:
            raise HTTPException(
                status_code=status.HTTP_404_NOT_FOUND,
                detail="User not found"
            )
        
        user_data['id'] = str(user_data['_id'])
        # Handle start_date
        if 'start_date' in user_data:
            if isinstance(user_data['start_date'], dict):
                user_data['start_date'] = user_data['start_date']['$date']
            if isinstance(user_data['start_date'], datetime):
                user_data['start_date'] = user_data['start_date'].isoformat()

        # Handle end_date
        if 'end_date' in user_data:
            if isinstance(user_data['end_date'], dict):
                user_data['end_date'] = user_data['end_date']['$date']
            if isinstance(user_data['end_date'], datetime):
                user_data['end_date'] = user_data['end_date'].isoformat()

        
        user_model = _User(**user_data)
        return user_model
    except Exception as e:
        traceback.print_exc()
        raise HTTPException(status_code=500, detail=str(e)) 
    
@u.post("/create/user")
async def create_user(user: _UserCreateRequest):
    user_data = mongo_database["Users"].find_one({"email": user.email})
    if user_data:
        return {"id": str(user_data["_id"])}

    user_data = user.dict()
    user_data.setdefault('user_companies_count', 0)
    user_data.setdefault('user_jobs_count', 0)
    user_data.setdefault('user_contacts_count', 0)
    user_data.setdefault('user_emails_count', 0)
    user_data.setdefault('user_can_send_emails', False)
    user_data.setdefault('created_at', datetime.now())
    user_data.setdefault('updated_at', datetime.now())
    user_data['start_date'] = datetime.now().isoformat() + "Z"
    user_data['end_date'] = (datetime.now() + timedelta(days=7)).isoformat() + "Z"  # Assuming free plan is monthly
    result = mongo_database["Users"].insert_one(user_data)

    # Create Stripe Customer
    API_KEY = os.getenv("CHARGEBEE_SECRET_KEY")
    SITE = os.getenv("CHARGEBEE_SITE")

    chargebee.configure("{API_KEY}","{SITE}")
    customer = chargebee.Customer.create(email=user.email)
    
    # Subscribe Customer to Free Plan
    # basic_plan=mongo_database["Plan"].find_one({"plan_type":"Basic"})
    # free_plan_id = os.getenv('STRIPE_FREE_PLAN_MONTHLY_ID')
    # free_plan_id=basic_plan["monthly_price_id"]
    # subscription = chargebee.Subscription.create_with_items(
    #     customer=customer.id,
    #     items=[{"plan": free_plan_id}]
    # )
    result = chargebee.Subscription.create_with_items(customer.id,{
        "subscription_items" : [
            {
                "item_price_id" : "FREE",
                "unit_price" : 100,
                "billing_cycles" : 1,
                "quantity" : 1
                
            }]
    })

    subscription = result.subscription
    customer = result.customer
    # Store the Stripe customer ID and subscription ID in MongoDB
    user_data['stripe_customer_id'] = customer.id
    user_data['stripe_subscription_id'] = subscription.id
    user_data['start_date'] = datetime.now().isoformat() + "Z"
    user_data['end_date'] = (datetime.now() + timedelta(days=7)).isoformat() + "Z"  # Assuming free plan is monthly
    
    mongo_database["Users"].update_one({"_id": result.inserted_id}, {"$set": user_data})

    return {"id": str(result.inserted_id)}

@u.get("/get-customer-id/{email}")
async def get_customer_id(email: str):
    user_data = mongo_database["Users"].find_one({"email": email})
    if not user_data:
        raise HTTPException(status_code=404, detail="User not found")

    stripe_customer_id = user_data.get("stripe_customer_id")
    if not stripe_customer_id:
        raise HTTPException(status_code=404, detail="Stripe customer ID not found")

    return {"stripe_customer_id": stripe_customer_id}

# Endpoint to create a new candidate
@u.post("/create/candidate")
async def create_candidate(
    data: _Contact,
):
    """
    Create a new candidate.

    :param data: Data of the candidate to be created.
    :type data: dict
    :return: ID of the created candidate.
    :rtype: dict
    """
    
    if data.email == "" or data.email == None:
        raise HTTPException(
            status_code=status.HTTP_400_BAD_REQUEST,
            detail="No candidate provided",
        )
    
    # check if user exists
    user_data = mongo_database["Candidate"].find_one({"email": data.email})
    if user_data:
        # raise HTTPException(
        #     status_code=status.HTTP_400_BAD_REQUEST,
        #     detail="Candidate already exists"
        # )
        return {"id": str(user_data["_id"])}

    response = mongo_database["Candidate"].insert_one(data.dict())

    return {"id": str(response.inserted_id)}


@u.get("/has_access")
async def has_access(email: str, entity_type: str):
    
    
    plan_collection = mongo_database["Plan"]
    user_collection = mongo_database["Users"]
    # Retrieve user's plan details
    user_plan = user_collection.find_one({"email": email})
    if user_plan is None:
        raise HTTPException(status_code=404, detail="User not found")

    plan = plan_collection.find_one({"plan_type": user_plan["plan"]})
    if plan is None:
        raise HTTPException(status_code=404, detail="Plan not found")

    # Retrieve entity count based on entity type
    # print(user_plan)
    entity_count = 0
    # print(entity_type.lower() )
    if entity_type.lower() == "company":
        entity_count = user_plan["user_companies_count"]
    elif entity_type.lower() == "job":
        entity_count = user_plan["user_jobs_count"]
    elif entity_type.lower() == "contact":
        entity_count = user_plan["user_contacts_count"]
    elif entity_type.lower() == "email":
        entity_count = user_plan["user_emails_count"]
    else:
        raise HTTPException(status_code=400, detail="Invalid entity type")
    
    # if (plan["max_companies_per_user"] == "Unlimited") and (plan["max_jobs"] == "Unlimited") and (plan["max_mobile_contacts_per_user"] == "Unlimited") and (plan["max_emails_per_user"] == "Unlimited"):
    #     return True
    if (plan["max_companies_per_user"] == "Unlimited" or plan["max_companies_per_user"] == -1) and \
        (plan["max_jobs"] == "Unlimited" or plan["max_jobs"] == -1) and \
        (plan["max_mobile_contacts_per_user"] == "Unlimited" or plan["max_mobile_contacts_per_user"] == -1) and \
        (plan["max_emails_per_user"] == "Unlimited" or plan["max_emails_per_user"] == -1):
        return True


    # Check if entity count exceeds the limit
    if entity_type.lower() == "company" and entity_count >= int(plan["max_companies_per_user"]):
        is_within_limit = False 
    elif entity_type.lower() == "job" and entity_count >= int(plan["max_jobs"]):
         is_within_limit = False
    elif entity_type.lower() == "contact" and entity_count >= int(plan["max_mobile_contacts_per_user"]):
        is_within_limit = False
    elif entity_type.lower() == "email" and entity_count >= int(plan["max_emails_per_user"]):
        is_within_limit = False
    else:
        is_within_limit = True

    return is_within_limit

@u.put("/increment/{entity_type}")
async def increment_entity_count(entity_type: str, email: str):

    users_collection = mongo_database["Users"]

    if entity_type.lower() not in ["contact", "email"]:
        raise HTTPException(status_code=400, detail="Invalid entity type")

    # Find the user document
    user = users_collection.find_one({"email": email})

    if not user:
        raise HTTPException(status_code=404, detail="User not found")

    # Increment the respective counter
    if entity_type.lower() == "contact":
        updated_count = user["user_contacts_count"] + 1
        users_collection.update_one({"email": email}, {"$set": {"user_contacts_count": updated_count,"updated_at": datetime.now()}})
    elif entity_type.lower() == "email":
        updated_count = user["user_emails_count"] + 1
        users_collection.update_one({"email": email}, {"$set": {"user_emails_count": updated_count,"updated_at": datetime.now()}})

    return {"message": "Counter incremented successfully"}
    
def calculate_end_date(start_date: datetime, frequency: str) -> datetime:
    if frequency == "monthly":
        return start_date + timedelta(days=30)
    elif frequency == "yearly":
        return start_date + timedelta(days=365)
    else:
        raise ValueError("Invalid frequency. Must be 'monthly' or 'yearly'.")

@u.post("/plan/subscribe")
async def create_user_plan(subscription: SubscriptionRequest):
    user_collection = mongo_database["Users"]
    user = user_collection.find_one({"_id": ObjectId(subscription.user_id)})
    if not user:
        raise HTTPException(status_code=404, detail="User not found")

    # Ensure valid plan type and frequency
    valid_plan_types = ["free", "starter", "premium", "enterprise"]
    if subscription.plan_type not in valid_plan_types:
        raise HTTPException(status_code=400, detail="Invalid plan type")
    
    valid_frequencies = ["monthly", "yearly"]
    if subscription.frequency not in valid_frequencies:
        raise HTTPException(status_code=400, detail="Invalid frequency")

    start_date = datetime.now()
    end_date = calculate_end_date(start_date, subscription.frequency)

    # Update the user document with the subscription details
    user_collection.update_one(
        {"_id": ObjectId(subscription.user_id)},
        {"$set": {
            "plan": subscription.plan_type,
            "start_date": start_date.isoformat() + "Z",
            "end_date": end_date.isoformat() + "Z",
            "updated_at": datetime.now()
        }}
    )

    return {
        "plan": subscription.plan_type,
        "start_date": start_date.isoformat() + "Z",
        "end_date": end_date.isoformat() + "Z"
    }


@u.post("/plan/renewal")
async def renew_plan(renewal: RenewalRequest):
    users_collection = mongo_database["Users"]
    user = users_collection.find_one({"_id": ObjectId(renewal.user_id)})
    if not user:
        raise HTTPException(status_code=404, detail="User not found")

    current_end_date_str = user.get("end_date")
    if not current_end_date_str:
        raise HTTPException(status_code=400, detail="User does not have an active plan")

    try:
        current_end_date = datetime.fromisoformat(current_end_date_str.rstrip("Z"))
    except ValueError:
        raise HTTPException(status_code=400, detail="Invalid end date format")

    # Calculate the new end date based on the frequency
    new_end_date = calculate_end_date(current_end_date, renewal.frequency)

    # Reset the counters and update the end date
    users_collection.update_one(
        {"_id": ObjectId(renewal.user_id)},
        {"$set": {
            "start_date": current_end_date,
            "end_date": new_end_date.isoformat() + "Z",
            "user_companies_count": 0,
            "user_jobs_count": 0,
            "user_contacts_count": 0,
            "user_emails_count": 0,
            "updated_at": datetime.now()
        }}
    )

    return {
        "user_id": renewal.user_id,
        "new_end_date": new_end_date.isoformat() + "Z"
    }

@u.post("/send/email")
async def send_Email(req:Request):
 
    response= await req.json()
    data=response["data"] 
    subject = response["subject"]

    support_email=os.environ["SUPPORT_EMAIL"]
    support_password=os.environ["SUPPORT_PASSWORD"]
    
    emails=response["email"]
    s = smtplib.SMTP('smtp.gmail.com', 587)
    s.starttls()
    s.login(support_email,support_password )
    message = data
    
    for dest in emails:
        dest=unquote(dest)
        message = f"Subject: {subject}\n\n{data}"
        s.sendmail(support_email, dest, message.encode('utf-8'))
  
    s.quit()
    
    return JSONResponse(status_code=200, content={"Detail":"Email Sent Successfully"})




import logging
import os
from llama_index.vector_stores.mongodb import MongoDBAtlasVectorSearch
from llama_index.core.indices import VectorStoreIndex

from app.engine.context import create_service_context
from app.engine.mongo import get_mongo_store

def get_index():
    try:
        logger = logging.getLogger("uvicorn")
        logger.info("Connecting to index from MongoDB...")
        # service_context = create_service_context()
        store = get_mongo_store()
        # print('store is :: ' + str(store.db_name))
        # store = get_mongo_store()
        index = VectorStoreIndex.from_vector_store(store)
       
        logger.info("Finished connecting to index from MongoDB.")
        return index
    except Exception as e:
        logger.error(f"Error while connecting to index: {e}")
        # Handle the error as required
        return None  # Or raise the exception again if needed
    

